'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

/**
 * @file define getter functions
 * @author wangyisheng@baidu.com (wangyisheng)
 */

const MIP_VIEW_ID = 'mip-router__view';

function getMIPTitle(rawContent) {
    let title;

    if (!rawContent) {
        let titleDom = document.querySelector('title');
        if (titleDom) {
            title = titleDom.innerHTML.trim();
        }
    }
    else {
        let match = rawContent.match(/<title>([\s\S]+)<\/title>/);
        if (match) {
            title = match[1];
        }
    }

    return title || '百度一下，你就知道';
}

function getMIPData(rawContent) {
    let rawData;

    if (!rawContent) {
        let dataDom = document.querySelector('mip-data');
        if (!dataDom) {
            return;
        }

        rawData = dataDom.innerHTML.trim();
    }
    else {
        let match = rawContent.match(/<mip-data>([\s\S]+)<\/mip-data>/);
        if (!match) {
            return;
        }

        rawData = match[1].trim();
    }

    return JSON.parse(rawData);
}

function getMIPContent(rawContent) {
    let rawResult;

    if (!rawContent) {
        let contentDom = document.querySelector('mip-content');

        if (!contentDom) {
            return '';
        }

        rawResult = contentDom.innerHTML.trim();
    }
    else {
        let match = rawContent.match(/<mip-content>([\s\S]+)<\/mip-content>/);

        if (!match) {
            return '';
        }

        rawResult = match[1].trim();
    }

    // Create a root node
    return `<div id="${MIP_VIEW_ID}">${rawResult}</div>`;
}

function getMIPComponents(rawContent) {
    // Eliminate duplicate scripts
    let componentsSet = new Set();

    if (!rawContent) {
        document.querySelectorAll('script').forEach(script => {
            let src = script.getAttribute('src') || '';
            let match = src.match(/\/mip-(.+)\.js$/);

            // filter mip-router.js out
            // TODO delete me when mip-router.js is combined with mip.js
            if (match && match !== 'router') {
                componentsSet.add(match[1]);
                componentsSrcMap[match[1]] = src;
            }
        });
    }
    else {
        let match = rawContent.match(/<script[\s\S]*?>/g);

        if (match) {
            match.forEach(scriptAttrStr => {
                let innerMatch = scriptAttrStr.match(/src=['"]?(.+\/mip-(.+?)\.js)/);

                if (innerMatch && innerMatch[2] !== 'router') {
                    componentsSet.add(innerMatch[2]);
                    componentsSrcMap[innerMatch[2]] = innerMatch[1];
                }
            });
        }
    }

    let components = [];
    componentsSet.forEach(com => components.push(com));
    return components;
}

/**
 * @file 处理组件相关
 * @author wangyisheng@baidu.com (wangyisheng)
 */

// Store name of loaded component scripts
// e.g. ['tree', 'hello']
let loadedComponents = [];

// Store map from components name to src
// e.g. {'tree': 'some.cdn.com/mip-components/mip-tree.js'}
let componentsSrcMap$1 = Object.create(null);
// Get names of components which need to be loaded
function getNewComponents(diffComponents) {
    let newComponents = [];

    diffComponents.forEach(diffCom => {
        if (loadedComponents.indexOf(diffCom) === -1) {
            newComponents.push(diffCom);
        }
    });

    return newComponents;
}

// Store names of components which has already been loaded
// Explictly used in first loading
function addLoadedComponents(newComponents) {
    loadedComponents = loadedComponents.concat(newComponents);
}

// Load components by names
// Some components need additional template. Find them and append it. See https://github.com/mip-project/mip/issues/14
async function loadScripts(components, targetHTML) {
    function loadScript(src) {
        return new Promise((resolve, reject) => {
            let script = document.createElement('script');
            script.type = 'text/javascript';
            script.onload = resolve;
            script.onerror = reject;
            script.src = src;
            document.body.appendChild(script);
        });
    }

    function loadTemplateScript(componentName, content) {
        let templateScript = document.createElement('script');
        templateScript.type = 'text/x-template';
        templateScript.id = `mip-${componentName}-template`;
        templateScript.innerHTML = content;
        document.body.appendChild(templateScript);
    }

    await Promise.all(components.map(com => {
        // find template script for some components
        let reg = new RegExp('<script[\\s\\S]*?id=[\'"]?mip-' + com + '-template[\\s\\S]*?>([\\s\\S]+?)</script>');
        let match = targetHTML.match(reg);
        if (match) {
            loadTemplateScript(com, match[1]);
        }

        return loadScript(componentsSrcMap$1[com])
    }));

    addLoadedComponents(components);
}

/**
 * @file main entry
 * @author wangyisheng@baidu.com (wangyisheng)
 */

exports.getMIPTitle = getMIPTitle;
exports.getMIPData = getMIPData;
exports.getMIPContent = getMIPContent;
exports.getMIPComponents = getMIPComponents;
exports.getNewComponents = getNewComponents;
exports.addLoadedComponents = addLoadedComponents;
exports.loadScripts = loadScripts;
